from __future__ import absolute_import

import cpp_pyqubo
from cpp_pyqubo import *
from pyqubo.utils.asserts import *
from pyqubo.utils.solver import *
from .array import *
from .logical_constraint import *
from .logic import *
from pyqubo.integer.integer import *
from pyqubo.integer.log_encoded_integer import *
from pyqubo.integer.one_hot_enc_integer import *
from pyqubo.integer.order_enc_integer import *
from pyqubo.integer.unary_encoded_integer import *



# Copyright 2018 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cpp_pyqubo import SubH, UserDefinedExpress, Binary


class NotConst(SubH):
    """Constraint: Not(a) = b.

    Args:
        a (:class:`Express`): expression to be binary
        
        b (:class:`Express`): expression to be binary
        
        label (str): label to identify the constraint
    
    Examples:
        In this example, when the binary variables satisfy the constraint,
        the energy is 0.0. On the other hand, when they break the constraint,
        the energy is 1.0 > 0.0.
        
        >>> from pyqubo import NotConst, Binary
        >>> a, b = Binary('a'), Binary('b')
        >>> exp = NotConst(a, b, 'not')
        >>> model = exp.compile()
        >>> model.energy({'a': 1, 'b': 0}, vartype='BINARY')
        0.0
        >>> model.energy({'a': 1, 'b': 1}, vartype='BINARY')
        1.0
    """

    def __init__(self, a, b, label):
        express = 2 * a * b - a - b + 1
        super().__init__(express, label)



class AndConst(SubH):
    """Constraint: AND(a, b) = c.

    Args:
        a (:class:`Express`): expression to be binary
        
        b (:class:`Express`): expression to be binary
        
        c (:class:`Express`): expression to be binary
        
        label (str): label to identify the constraint
    
    Examples:
        In this example, when the binary variables satisfy the constraint,
        the energy is 0.0. On the other hand, when they break the constraint,
        the energy is 1.0 > 0.0.
        
        >>> from pyqubo import AndConst, Binary
        >>> a, b, c = Binary('a'), Binary('b'), Binary('c')
        >>> exp = AndConst(a, b, c, 'and')
        >>> model = exp.compile()
        >>> model.energy({'a': 1, 'b': 0, 'c': 0}, vartype='BINARY')
        0.0
        >>> model.energy({'a': 0, 'b': 1, 'c': 1}, vartype='BINARY')
        1.0
    """

    def __init__(self, a, b, c, label):
        express = a * b - 2 * (a + b) * c + 3 * c
        super().__init__(express, label)
    

class OrConst(SubH):
    """Constraint: OR(a, b) = c.

    Args:
        a (:class:`Express`): expression to be binary
        
        b (:class:`Express`): expression to be binary
        
        c (:class:`Express`): expression to be binary
        
        label (str): label to identify the constraint
    
    Examples:
        In this example, when the binary variables satisfy the constraint,
        the energy is 0.0. On the other hand, when they break the constraint,
        the energy is 1.0 > 0.0.
        
        >>> from pyqubo import OrConst, Binary
        >>> a, b, c = Binary('a'), Binary('b'), Binary('c')
        >>> exp = OrConst(a, b, c, 'or')
        >>> model = exp.compile()
        >>> model.energy({'a': 1, 'b': 0, 'c': 1}, vartype='BINARY')
        0.0
        >>> model.energy({'a': 0, 'b': 1, 'c': 0}, vartype='BINARY')
        1.0
    """

    def __init__(self, a, b, c, label):
        express = a * b + (a + b) * (1 - 2 * c) + c
        super().__init__(express, label)
    

class XorConst(SubH):
    """Constraint: OR(a, b) = c.

    Args:
        a (:class:`Express`): expression to be binary
        
        b (:class:`Express`): expression to be binary
        
        c (:class:`Express`): expression to be binary
        
        label (str): label to identify the constraint
    
    Examples:
        In this example, when the binary variables satisfy the constraint,
        the energy is 0.0. On the other hand, when they break the constraint,
        the energy is 1.0 > 0.0.
        
        >>> from pyqubo import XorConst, Binary
        >>> a, b, c = Binary('a'), Binary('b'), Binary('c')
        >>> exp = XorConst(a, b, c, 'xor')
        >>> model = exp.compile()
        >>> model.energy({'a': 1, 'b': 0, 'c': 1, 'aux_xor': 0}, vartype='BINARY')
        0.0
        >>> model.energy({'a': 0, 'b': 1, 'c': 0, 'aux_xor': 0}, vartype='BINARY')
        1.0
    """

    def __init__(self, a, b, c, label):
        aux = Binary("aux_" + label)
        express = 2 * a * b - 2 * (a + b) * c - 4 * (a + b) * aux +\
                                4 * aux * c + a + b + c + 4 * aux

        super().__init__(express, label)



# Copyright 2018 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pyqubo.array import Array
from pyqubo.integer import IntegerWithPenalty
from cpp_pyqubo import Placeholder, Constraint, SubH


class OrderEncInteger(IntegerWithPenalty):
    """
    Order encoded integer. This encoding is useful when you want to know
    whether the integer is more than k or not.
    The value that takes :math:`[0, n]` is represented by :math:`\sum_{i=1}^{n}x_{i}`.
    Also we have the penalty function :math:`strength \\times \\left(\sum_{i=1}^{n-1}\
    \\left(x_{i+1}-x_{i}x_{i+1}\\right)\\right)` in the Hamiltonian.
    See the reference [TaTK09]_ for more details.
    
    Args:
        label (str): Label of the integer.
        
        lower (int): Lower value of the integer.
        
        upper (int): Upper value of the integer.
        
        strength (float/Placeholder): Strength of the constraint.
    
    Examples:
        Create an order encoded integer `a` that takes [0, 3] with the strength = 5.0.
        Solution of `a` represents 2 which is the optimal solution of the Hamiltonian.
        
        >>> from pyqubo import OrderEncInteger
        >>> a = OrderEncInteger("a", (0, 3), strength = 5.0)
        >>> model = ((a-2)**2).compile()
        >>> bqm = model.to_bqm()
        >>> import dimod
        >>> sampleset = dimod.ExactSolver().sample(bqm)
        >>> decoded_samples = model.decode_sampleset(sampleset)
        >>> best_sample = min(decoded_samples, key=lambda s: s.energy)
        >>> print(best_sample.subh['a'])
        2.0
    """

    def __init__(self, label, value_range, strength):
        lower, upper = value_range
        assert upper > lower, "upper value should be larger than lower value"
        assert isinstance(lower, int)
        assert isinstance(upper, int)
        assert isinstance(strength, int) or isinstance(strength, float) or\
            isinstance(strength, Placeholder)

        self._num_variables = (upper - lower)
        self.array = Array.create(label, shape=self._num_variables, vartype='BINARY')

        self.constraint = 0.0
        for i in range(self._num_variables - 1):
            a = self.array[i]
            b = self.array[i + 1]
            const_label = label + "_order_" + str(i)
            self.constraint += Constraint(b-a*b, const_label, condition=lambda x: x==0)

        express = SubH(lower + sum(self.array), label=label)
        penalty = self.constraint * strength

        super().__init__(
            label=label,
            value_range=value_range,
            express=express,
            penalty=penalty)


    def more_than(self, k):
        """Binary variable that represents whether the value is more than `k`.
        
        Note:
            You cannot use this method alone. You should use this variable with the entire integer.
            See an example below.

        Args:
            k (int): Integer value.
        
        Returns:
             :class:`Express`
        
        Examples:
            This example finds the value of integer `a` and `b` such that
            :math:`a=b` and :math:`a>1` and :math:`b<3`.
            The obtained solution is :math:`a=b=2`.
            
            >>> from pyqubo import OrderEncInteger
            >>> a = OrderEncInteger("a", (0, 4), strength = 5.0)
            >>> b = OrderEncInteger("b", (0, 4), strength = 5.0)
            >>> model = ((a-b)**2 + (1-a.more_than(1))**2 + (1-b.less_than(3))**2).compile()
            >>> bqm = model.to_bqm()
            >>> import dimod
            >>> sampleset = dimod.ExactSolver().sample(bqm)
            >>> decoded_samples = model.decode_sampleset(sampleset)
            >>> best_sample = min(decoded_samples, key=lambda s: s.energy)
            >>> print(best_sample.subh['a'])
            2.0
            >>> print(best_sample.subh['b'])
            2.0
        """
        assert isinstance(k, int), "k should be integer"
        lower, upper = self.value_range
        assert k > lower, "This value is always equal to or more than {}".format(k)
        assert k <= upper, "This value is never more than {}".format(k)
        return self.array[k-lower]

    def less_than(self, k):
        """Binary variable that represents whether the value is less than `k`.

        Note:
            You cannot use this method alone. You should use this variable with the entire integer.
            See an example below.

        Args:
            k (int): Integer value.

        Returns:
             :class:`Express`

        Examples:
            This example finds the value of integer `a` and `b` such that
            :math:`a=b` and :math:`a>1` and :math:`b<3`.
            The obtained solution is :math:`a=b=2`.
            
            >>> from pyqubo import OrderEncInteger
            >>> a = OrderEncInteger("a", (0, 4), strength = 5.0)
            >>> b = OrderEncInteger("b", (0, 4), strength = 5.0)
            >>> model = ((a-b)**2 + (1-a.more_than(1))**2 + (1-b.less_than(3))**2).compile()
            >>> bqm = model.to_bqm()
            >>> import dimod
            >>> sampleset = dimod.ExactSolver().sample(bqm)
            >>> decoded_samples = model.decode_sampleset(sampleset)
            >>> best_sample = min(decoded_samples, key=lambda s: s.energy)
            >>> print(best_sample.subh['a'])
            2.0
            >>> print(best_sample.subh['b'])
            2.0
        """
        assert isinstance(k, int), "k should be integer"
        lower, upper = self.value_range
        assert k >= lower, "This value is always more than {}".format(k)
        assert k < upper, "This value is always more than {}".format(k)
        return 1-self.array[k-lower-1]



# (major, minor, patch, prerelease)

VERSION = (1, 0, 5, "")
__shortversion__ = '.'.join(map(str, VERSION[:3]))
__version__ = '.'.join(map(str, VERSION[:3])) + "".join(VERSION[3:])

__package_name__ = 'pyqubo'
__contact_names__ = 'Recruit Communications Co., Ltd.'
__contact_emails__ = 'rco_pyqubo@ml.cocorou.jp'
__homepage__ = 'https://pyqubo.readthedocs.io/en/latest/'
__repository_url__ = 'https://github.com/recruit-communications/pyqubo'
__download_url__ = 'https://github.com/recruit-communications/pyqubo'
__description__ = 'PyQUBO allows you to create QUBOs or Ising models from mathematical expressions.'
__license__ = 'Apache 2.0'
__keywords__ = 'QUBO, quantum annealing, annealing machine, ising model, optimization'



# Copyright 2020 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc
from cpp_pyqubo import UserDefinedExpress, WithPenalty


class Integer(UserDefinedExpress):
    """Binary encoded integer
    """

    def __init__(self, label, value_range, express):
        self.label=label
        self.value_range = value_range
        super().__init__(express)
    
    def __str__(self):
        return "{}({},value_range={})".format(type(self).__name__, self.label, self.value_range)


class IntegerWithPenalty(WithPenalty):
    """Binary encoded integer with penalty function
    """

    def __init__(self, label, value_range, express, penalty):
        self.label=label
        self.value_range = value_range
        super().__init__(express, penalty, label)
    
    def __str__(self):
        return "{}({},value_range={})".format(type(self).__name__, self.label, self.value_range)



# Copyright 2018 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pyqubo import Array, SubH, Constraint
from pyqubo.integer.integer import IntegerWithPenalty
from pyqubo import WithPenalty, Placeholder


class OneHotEncInteger(IntegerWithPenalty):
    """One-hot encoded integer. The value that takes :math:`[1, n]` is represented by :math:`\sum_{i=1}^{n}ix_{i}`.
    Also we have the penalty function :math:`strength \\times (\sum_{i=1}^{n}x_{i}-1)^2` in the Hamiltonian.
    
    Args:
        label (str): Label of the integer.
        
        lower (int): Lower value of the integer.
        
        upper (int): Upper value of the integer.
        
        strength (float/Placeholder): Strength of the constraint.
    
    Examples:
        This example is equivalent to the following Hamiltonian.
        
        .. math::
        
            H = \\left(\\left(\sum_{i=1}^{3}ia_{i}+1\\right) - 2\\right)^2 + strength \\times \\left(\sum_{i=1}^{3}a_{i}-1\\right)^2
        
        >>> from pyqubo import OneHotEncInteger
        >>> a = OneHotEncInteger("a", (1, 3), strength=5)
        >>> H = (a-2)**2
        >>> model = H.compile()
        >>> bqm = model.to_bqm()
        >>> import dimod
        >>> sampleset = dimod.ExactSolver().sample(bqm)
        >>> decoded_samples = model.decode_sampleset(sampleset)
        >>> best_sample = min(decoded_samples, key=lambda s: s.energy)
        >>> print(best_sample.subh['a'])
        2.0
    """

    def __init__(self, label, value_range, strength):
        lower, upper = value_range
        assert upper > lower, "upper value should be larger than lower value"
        assert isinstance(lower, int)
        assert isinstance(upper, int)
        assert isinstance(strength, int) or isinstance(strength, float) or\
            isinstance(strength, Placeholder)

        self._num_variables = (upper - lower + 1)
        self.array = Array.create(label, shape=self._num_variables, vartype='BINARY')
        self.constraint = Constraint((sum(self.array)-1)**2, label=label+"_const", condition=lambda x: x==0)

        express = SubH(lower + sum(i*x for i, x in enumerate(self.array)), label=label)
        penalty = self.constraint * strength

        super().__init__(
            label=label,
            value_range=value_range,
            express=express,
            penalty=penalty)

    def equal_to(self, k):
        """Variable representing whether the value is equal to `k`.
        
        Note:
            You cannot use this method alone. You should use this variable with the entire integer.
    
        Args:
            k (int): Integer value. 
        
        Returns:
            :class:`Express`
        """
        lower, upper = self.value_range
        assert isinstance(k, int), "k should be integer"
        assert lower <= k <= upper, "This value never takes {}".format(k)
        return self.array[k-lower]




# Copyright 2018 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cpp_pyqubo import UserDefinedExpress


class Not(UserDefinedExpress):
    """Logical NOT of input.

    Args:
        bit (:class:`Express`): expression to be binary
    
    Examples:
        >>> from pyqubo import Binary, Not
        >>> a = Binary('a')
        >>> exp = Not(a)
        >>> model = exp.compile()
        >>> for a in (0, 1):
        ...   print(a, int(model.energy({'a': a}, vartype='BINARY')))
        0 1
        1 0
    """

    def __init__(self, bit):
        hamiltonian = 1 - bit
        super().__init__(hamiltonian)


class And(UserDefinedExpress):
    """Logical AND of inputs.

    Args:
        bit_a (:class:`Express`): expression to be binary
        
        bit_b (:class:`Express`): expression to be binary
    
    Examples:
        >>> from pyqubo import Binary, And
        >>> import itertools
        >>> a, b = Binary('a'), Binary('b')
        >>> exp = And(a, b)
        >>> model = exp.compile()
        >>> for a, b in itertools.product(*[(0, 1)] * 2):
        ...   print(a, b, int(model.energy({'a': a, 'b': b}, vartype='BINARY')))
        0 0 0
        0 1 0
        1 0 0
        1 1 1
    """

    def __init__(self, bit_a, bit_b):
        hamiltonian = bit_a * bit_b
        super().__init__(hamiltonian)


class Or(UserDefinedExpress):
    """Logical OR of inputs.

    Args:
        bit_a (:class:`Express`): expression to be binary
        bit_b (:class:`Express`): expression to be binary
    
    Examples:
        
        >>> from pyqubo import Binary, Or
        >>> import itertools
        >>> a, b = Binary('a'), Binary('b')
        >>> exp = Or(a, b)
        >>> model = exp.compile()
        >>> for a, b in itertools.product(*[(0, 1)] * 2):
        ...   print(a, b, int(model.energy({'a': a, 'b': b}, vartype='BINARY')))
        0 0 0
        0 1 1
        1 0 1
        1 1 1
    """

    def __init__(self, bit_a, bit_b):
        hamiltonian = Not(And(Not(bit_a), Not(bit_b)))
        super().__init__(hamiltonian)


class Xor(UserDefinedExpress):
    """Logical XOR of inputs.

    Args:
        bit_a (:class:`Express`): expression to be binary
        bit_b (:class:`Express`): expression to be binary

    Examples:

        >>> from pyqubo import Binary, Xor
        >>> import itertools
        >>> a, b = Binary('a'), Binary('b')
        >>> exp = Xor(a, b)
        >>> model = exp.compile()
        >>> for a, b in itertools.product(*[(0, 1)] * 2):
        ...   print(a, b, int(model.energy({'a': a, 'b': b}, vartype='BINARY')))
        0 0 0
        0 1 1
        1 0 1
        1 1 0
    """

    def __init__(self, bit_a, bit_b):
        nand_ab = Not(And(bit_a, bit_b))
        or_ab = Or(bit_a, bit_b)
        hamiltonian = And(nand_ab, or_ab)
        super().__init__(hamiltonian)



# Copyright 2018 Recruit Communications Co., Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from cpp_pyqubo import SubH
from pyqubo.array import Array
from pyqubo.integer import Integer
import numpy as np


class LogEncInteger(Integer):
    """Log encoded integer. The value that takes :math:`[0, n]` is
    represented by :math:`\\sum_{i=1}^{\\lceil\\log_{2}n\\rceil}2^ix_{i}` without any constraint.

    Args:
        label (str): Label of the integer.

        lower (int): Lower value of the integer.

        upper (int): Upper value of the integer.

    Examples:
        This example finds the value `a`, `b` such that :math:`a+b=5` and :math:`2a-b=1`.
        
        >>> from pyqubo import LogEncInteger
        >>> import dimod
        >>> a = LogEncInteger("a", (0, 4))
        >>> b = LogEncInteger("b", (0, 4))
        >>> M=2.0
        >>> H = (2*a-b-1)**2 + M*(a+b-5)**2
        >>> model = H.compile()
        >>> bqm = model.to_bqm()
        >>> import dimod
        >>> sampleset = dimod.ExactSolver().sample(bqm)
        >>> decoded_samples = model.decode_sampleset(sampleset)
        >>> best_sample = min(decoded_samples, key=lambda s: s.energy)
        >>> print(best_sample.subh['a'])
        2.0
        >>> print(best_sample.subh['b'])
        3.0
    """

    def __init__(self, label, value_range):
        lower, upper = value_range
        assert upper > lower, "upper value should be larger than lower value"
        assert isinstance(lower, int)
        assert isinstance(upper, int)

        span = upper - lower
        
        self._num_variables = int(np.log2(span)) + 1
        self.array = Array.create(label, shape=self._num_variables, vartype='BINARY')
        d = self._num_variables - 1
        express = lower + sum(self.array[i] * 2 ** i for i in range(self._num_variables - 1))
        express += (span - (2**d - 1)) * self.array[-1]
        express = SubH(express, label)
        
        super().__init__(
            label=label,
            value_range=value_range,
            express=express)



import os
import re
import sys
import platform
import subprocess
import sysconfig

from setuptools import Extension
from setuptools.command.build_ext import build_ext
from distutils.version import LooseVersion
from setuptools import Command


if os.getenv('READTHEDOCS'):
    from skbuild import setup
else:
    from setuptools import setup


class PackageInfo(object):
    def __init__(self, info_file):
        with open(info_file) as f:
            exec(f.read(), self.__dict__)
        self.__dict__.pop('__builtins__', None)

    def __getattribute__(self, name):
        return super(PackageInfo, self).__getattribute__(name)

package_info = PackageInfo(os.path.join('pyqubo', 'package_info.py'))

class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError("CMake must be installed to build the following extensions: " +
                               ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)', out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(cfg.upper(), extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j2']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(env.get('CXXFLAGS', ''),
                                                              self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args, cwd=self.build_temp)


class CppTest(Command):

    def initialize_options(self):
        self.cpplibdir = self.distutils_dir_name()

    def finalize_options(self):
        pass

    user_options = []

    def distutils_dir_name(self):
        """Returns the name of a distutils build directory"""
        f = "temp.{platform}-{version[0]}.{version[1]}"
        return f.format(platform=sysconfig.get_platform(),
                        version=sys.version_info)

    def run(self):
        subprocess.call(['make pyqubo_test'],
                        cwd=os.path.join('build', self.cpplibdir), shell=True)
        subprocess.call(['./tests/pyqubo_test'],
                        cwd=os.path.join('build', self.cpplibdir), shell=True)

packages = ['pyqubo', 'pyqubo.integer', 'pyqubo.utils']

install_requires = [
    'numpy>=1.16.0,<2.0.0',
    'dimod>=0.9.2',
    'dwave-neal>=0.5.4',
    'Deprecated>=1.2.10',
    'six>=1.11.0'
    ]

python_requires = '>=3.5'

setup(
    name=package_info.__package_name__,
    version=package_info.__version__,
    description=package_info.__description__,
    long_description=open('README.rst').read(),
    author=package_info.__contact_names__,
    author_email=package_info.__contact_emails__,
    maintainer=package_info.__contact_names__,
    maintainer_email=package_info.__contact_emails__,
    url=package_info.__repository_url__,
    download_url=package_info.__download_url__,
    license=package_info.__license__,
    ext_modules=[CMakeExtension('pyqubo')],
    cmdclass=dict(build_ext=CMakeBuild, cpp_test=CppTest),
    zip_safe=False,
    packages=packages,
    keywords=package_info.__keywords__,
    install_requires=install_requires,
    python_requires=python_requires,
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'License :: OSI Approved :: Apache Software License',
    ]
)
